@@ -34,12 +34,37 @@ const TYPE_FLOAT: u8 = 15;
 /// This matches the value used in libmaxminddb and the Go reader.
 const MAXIMUM_DATA_STRUCTURE_DEPTH: u16 = 512;
 
+#[inline(always)]
 fn to_usize(base: u8, bytes: &[u8]) -> usize {
     bytes
         .iter()
         .fold(base as usize, |acc, &b| (acc << 8) | b as usize)
 }
 
+macro_rules! decode_int_like {
+    ($name:ident, $ty:ty, $max_size:expr, $label:literal, $zero:expr) => {
+        fn $name(&mut self, size: usize) -> DecodeResult<$ty> {
+            match size {
+                s if s <= $max_size => {
+                    let new_offset = self
+                        .current_ptr
+                        .checked_add(size)
+                        .filter(|&offset| offset <= self.buf.len())
+                        .ok_or_else(|| {
+                            self.invalid_db_error(&format!("{} of size {}", $label, size))
+                        })?;
+                    let value = self.buf[self.current_ptr..new_offset]
+                        .iter()
+                        .fold($zero, |acc, &b| (acc << 8) | <$ty>::from(b));
+                    self.current_ptr = new_offset;
+                    Ok(value)
+                }
+                s => Err(self.invalid_db_error(&format!("{} of size {}", $label, s))),
+            }
+        }
+    };
+}
+
 enum Value<'a, 'de> {
     Any { prev_ptr: usize },
     Bytes(&'de [u8]),
@@ -112,44 +137,59 @@ impl<'de> Decoder<'de> {
     }
 
     #[inline(always)]
-    fn eat_byte(&mut self) -> u8 {
-        let b = self.buf[self.current_ptr];
-        self.current_ptr += 1;
-        b
+    fn checked_offset(&self, size: usize, label: &str) -> DecodeResult<usize> {
+        let new_offset = self.current_ptr.wrapping_add(size);
+        if new_offset < self.current_ptr || new_offset > self.buf.len() {
+            return Err(self.invalid_db_error(&format!("{label} of size {size}")));
+        }
+        Ok(new_offset)
     }
 
     #[inline(always)]
-    fn size_from_ctrl_byte(&mut self, ctrl_byte: u8, type_num: u8) -> usize {
+    fn eat_byte(&mut self) -> DecodeResult<u8> {
+        let b = *self
+            .buf
+            .get(self.current_ptr)
+            .ok_or_else(|| self.invalid_db_error("unexpected end of buffer"))?;
+        self.current_ptr += 1;
+        Ok(b)
+    }
+
+    #[inline(always)]
+    fn size_from_ctrl_byte(&mut self, ctrl_byte: u8, type_num: u8) -> DecodeResult<usize> {
         let size = (ctrl_byte & 0x1f) as usize;
         // Extended type - size field is used differently
         if type_num == TYPE_EXTENDED {
-            return size;
+            return Ok(size);
         }
 
-        let bytes_to_read = size.saturating_sub(28);
-
-        let new_offset = self.current_ptr + bytes_to_read;
-        let size_bytes = &self.buf[self.current_ptr..new_offset];
-        self.current_ptr = new_offset;
-
         match size {
-            s if s < 29 => s,
-            29 => 29_usize + size_bytes[0] as usize,
-            30 => 285_usize + to_usize(0, size_bytes),
-            _ => 65_821_usize + to_usize(0, size_bytes),
+            s if s < 29 => Ok(s),
+            29 => Ok(29_usize + self.eat_byte()? as usize),
+            30 => {
+                let b0 = self.eat_byte()? as usize;
+                let b1 = self.eat_byte()? as usize;
+                Ok(285_usize + (b0 << 8) + b1)
+            }
+            _ => {
+                let b0 = self.eat_byte()? as usize;
+                let b1 = self.eat_byte()? as usize;
+                let b2 = self.eat_byte()? as usize;
+                Ok(65_821_usize + (b0 << 16) + (b1 << 8) + b2)
+            }
         }
     }
 
     #[inline(always)]
-    fn size_and_type(&mut self) -> (usize, u8) {
-        let ctrl_byte = self.eat_byte();
+    fn size_and_type(&mut self) -> DecodeResult<(usize, u8)> {
+        let ctrl_byte = self.eat_byte()?;
         let mut type_num = ctrl_byte >> 5;
         // Extended type: type 0 means read next byte for actual type
         if type_num == TYPE_EXTENDED {
-            type_num = self.eat_byte() + TYPE_MAP; // Extended types start at 7
+            type_num = self.eat_byte()? + TYPE_MAP; // Extended types start at 7
         }
-        let size = self.size_from_ctrl_byte(ctrl_byte, type_num);
-        (size, type_num)
+        let size = self.size_from_ctrl_byte(ctrl_byte, type_num)?;
+        Ok((size, type_num))
     }
 
     fn decode_any<V: Visitor<'de>>(&mut self, visitor: V) -> DecodeResult<V::Value> {
@@ -188,7 +228,7 @@ impl<'de> Decoder<'de> {
 
     #[inline(always)]
     fn decode_any_value(&mut self) -> DecodeResult<Value<'_, 'de>> {
-        let (size, type_num) = self.size_and_type();
+        let (size, type_num) = self.size_and_type()?;
 
         Ok(match type_num {
             TYPE_POINTER => {
@@ -235,7 +275,7 @@ impl<'de> Decoder<'de> {
     }
 
     fn decode_bytes(&mut self, size: usize) -> DecodeResult<&'de [u8]> {
-        let new_offset = self.current_ptr + size;
+        let new_offset = self.checked_offset(size, "bytes")?;
         let u8_slice = &self.buf[self.current_ptr..new_offset];
         self.current_ptr = new_offset;
 
@@ -243,7 +283,7 @@ impl<'de> Decoder<'de> {
     }
 
     fn decode_float(&mut self, size: usize) -> DecodeResult<f32> {
-        let new_offset = self.current_ptr + size;
+        let new_offset = self.checked_offset(size, "float")?;
         let value: [u8; 4] = self.buf[self.current_ptr..new_offset]
             .try_into()
             .map_err(|_| self.invalid_db_error(&format!("float of size {size}")))?;
@@ -253,7 +293,7 @@ impl<'de> Decoder<'de> {
     }
 
     fn decode_double(&mut self, size: usize) -> DecodeResult<f64> {
-        let new_offset = self.current_ptr + size;
+        let new_offset = self.checked_offset(size, "double")?;
         let value: [u8; 8] = self.buf[self.current_ptr..new_offset]
             .try_into()
             .map_err(|_| self.invalid_db_error(&format!("double of size {size}")))?;
@@ -262,79 +302,67 @@ impl<'de> Decoder<'de> {
         Ok(float_value)
     }
 
-    fn decode_uint64(&mut self, size: usize) -> DecodeResult<u64> {
-        match size {
-            s if s <= 8 => {
-                let new_offset = self.current_ptr + size;
+    decode_int_like!(decode_uint64, u64, 8, "u64", 0_u64);
+    decode_int_like!(decode_uint128, u128, 16, "u128", 0_u128);
 
-                let value = self.buf[self.current_ptr..new_offset]
-                    .iter()
-                    .fold(0_u64, |acc, &b| (acc << 8) | u64::from(b));
-                self.current_ptr = new_offset;
-                Ok(value)
-            }
-            s => Err(self.invalid_db_error(&format!("u64 of size {s}"))),
-        }
-    }
-
-    fn decode_uint128(&mut self, size: usize) -> DecodeResult<u128> {
-        match size {
-            s if s <= 16 => {
-                let new_offset = self.current_ptr + size;
-
-                let value = self.buf[self.current_ptr..new_offset]
-                    .iter()
-                    .fold(0_u128, |acc, &b| (acc << 8) | u128::from(b));
-                self.current_ptr = new_offset;
-                Ok(value)
-            }
-            s => Err(self.invalid_db_error(&format!("u128 of size {s}"))),
+    #[inline(always)]
+    fn read_u32_be(&mut self, size: usize, label: &str) -> DecodeResult<u32> {
+        if size > 4 {
+            return Err(self.invalid_db_error(&format!("{label} of size {size}")));
         }
+        let new_offset = self
+            .current_ptr
+            .checked_add(size)
+            .filter(|&offset| offset <= self.buf.len())
+            .ok_or_else(|| self.invalid_db_error(&format!("{label} of size {}", size)))?;
+        let p = self.current_ptr;
+        let value = match size {
+            0 => 0,
+            1 => self.buf[p] as u32,
+            2 => ((self.buf[p] as u32) << 8) | self.buf[p + 1] as u32,
+            3 => {
+                ((self.buf[p] as u32) << 16)
+                    | ((self.buf[p + 1] as u32) << 8)
+                    | self.buf[p + 2] as u32
+            }
+            _ => {
+                ((self.buf[p] as u32) << 24)
+                    | ((self.buf[p + 1] as u32) << 16)
+                    | ((self.buf[p + 2] as u32) << 8)
+                    | self.buf[p + 3] as u32
+            }
+        };
+        self.current_ptr = new_offset;
+        Ok(value)
     }
 
+    #[inline(always)]
     fn decode_uint32(&mut self, size: usize) -> DecodeResult<u32> {
-        match size {
-            s if s <= 4 => {
-                let new_offset = self.current_ptr + size;
-
-                let value = self.buf[self.current_ptr..new_offset]
-                    .iter()
-                    .fold(0_u32, |acc, &b| (acc << 8) | u32::from(b));
-                self.current_ptr = new_offset;
-                Ok(value)
-            }
-            s => Err(self.invalid_db_error(&format!("u32 of size {s}"))),
-        }
+        self.read_u32_be(size, "u32")
     }
 
+    #[inline(always)]
     fn decode_uint16(&mut self, size: usize) -> DecodeResult<u16> {
-        match size {
-            s if s <= 2 => {
-                let new_offset = self.current_ptr + size;
-
-                let value = self.buf[self.current_ptr..new_offset]
-                    .iter()
-                    .fold(0_u16, |acc, &b| (acc << 8) | u16::from(b));
-                self.current_ptr = new_offset;
-                Ok(value)
-            }
-            s => Err(self.invalid_db_error(&format!("u16 of size {s}"))),
+        if size > 2 {
+            return Err(self.invalid_db_error(&format!("u16 of size {size}")));
         }
+        let new_offset = self
+            .current_ptr
+            .checked_add(size)
+            .filter(|&offset| offset <= self.buf.len())
+            .ok_or_else(|| self.invalid_db_error(&format!("u16 of size {}", size)))?;
+        let p = self.current_ptr;
+        let value = match size {
+            0 => 0,
+            1 => self.buf[p] as u16,
+            _ => ((self.buf[p] as u16) << 8) | self.buf[p + 1] as u16,
+        };
+        self.current_ptr = new_offset;
+        Ok(value)
     }
 
     fn decode_int(&mut self, size: usize) -> DecodeResult<i32> {
-        match size {
-            s if s <= 4 => {
-                let new_offset = self.current_ptr + size;
-
-                let value = self.buf[self.current_ptr..new_offset]
-                    .iter()
-                    .fold(0_i32, |acc, &b| (acc << 8) | i32::from(b));
-                self.current_ptr = new_offset;
-                Ok(value)
-            }
-            s => Err(self.invalid_db_error(&format!("i32 of size {s}"))),
-        }
+        self.read_u32_be(size, "i32").map(|value| value as i32)
     }
 
     fn decode_map(&mut self, size: usize) -> Value<'_, 'de> {
@@ -344,11 +372,22 @@ impl<'de> Decoder<'de> {
         })
     }
 
+    #[inline(always)]
     fn decode_pointer(&mut self, size: usize) -> usize {
         let pointer_value_offset = [0, 0, 2048, 526_336, 0];
         let pointer_size = ((size >> 3) & 0x3) + 1;
-        let new_offset = self.current_ptr + pointer_size;
-        let pointer_bytes = &self.buf[self.current_ptr..new_offset];
+        let p = self.current_ptr;
+        let len = self.buf.len();
+        let new_offset = match p.checked_add(pointer_size) {
+            Some(offset) if offset <= len => offset,
+            _ => {
+                // Clamp to the end of the buffer so the next decode step fails
+                // with a normal bounds error instead of panicking here.
+                self.current_ptr = len;
+                return len;
+            }
+        };
+        let pointer_bytes = &self.buf[p..new_offset];
         self.current_ptr = new_offset;
 
         let base = if pointer_size == 4 {
@@ -365,7 +404,7 @@ impl<'de> Decoder<'de> {
     fn decode_string(&mut self, size: usize) -> DecodeResult<&'de str> {
         use std::str::from_utf8_unchecked;
 
-        let new_offset: usize = self.current_ptr + size;
+        let new_offset = self.checked_offset(size, "string")?;
         let bytes = &self.buf[self.current_ptr..new_offset];
         self.current_ptr = new_offset;
         // SAFETY:
@@ -384,10 +423,17 @@ impl<'de> Decoder<'de> {
         use simdutf8::basic::from_utf8;
         #[cfg(not(feature = "simdutf8"))]
         use std::str::from_utf8;
+        use std::str::from_utf8_unchecked;
 
-        let new_offset: usize = self.current_ptr + size;
+        let new_offset = self.checked_offset(size, "string")?;
         let bytes = &self.buf[self.current_ptr..new_offset];
         self.current_ptr = new_offset;
+        if bytes.is_ascii() {
+            // ASCII is valid UTF-8, so this avoids the full validator fast path.
+            // SAFETY: `is_ascii()` guarantees UTF-8 validity.
+            let v = unsafe { from_utf8_unchecked(bytes) };
+            return Ok(v);
+        }
         match from_utf8(bytes) {
             Ok(v) => Ok(v),
             Err(_) => Err(self.invalid_db_error("invalid UTF-8 in string")),
@@ -407,79 +453,139 @@ impl<'de> Decoder<'de> {
 
     /// Consumes a map header, returning its size. Follows pointers.
     pub(crate) fn consume_map_header(&mut self) -> DecodeResult<usize> {
-        let (size, type_num) = self.size_and_type();
-        if type_num == TYPE_POINTER {
-            let new_ptr = self.decode_pointer(size);
-            self.current_ptr = new_ptr;
-            self.consume_map_header()
-        } else if type_num == TYPE_MAP {
-            Ok(size)
-        } else {
-            Err(self.decode_error(&format!("expected map, got type {type_num}")))
-        }
+        self.consume_typed_header(TYPE_MAP, "map")
     }
 
     /// Consumes an array header, returning its size. Follows pointers.
     pub(crate) fn consume_array_header(&mut self) -> DecodeResult<usize> {
-        let (size, type_num) = self.size_and_type();
+        self.consume_typed_header(TYPE_ARRAY, "array")
+    }
+
+    /// Consumes a header of the expected type, following one pointer.
+    fn consume_typed_header(&mut self, expected_type: u8, label: &str) -> DecodeResult<usize> {
+        let (size, type_num) = self.size_and_type()?;
         if type_num == TYPE_POINTER {
-            let new_ptr = self.decode_pointer(size);
-            self.current_ptr = new_ptr;
-            self.consume_array_header()
-        } else if type_num == TYPE_ARRAY {
+            self.current_ptr = self.decode_pointer(size);
+            let (size, type_num) = self.size_and_type()?;
+            if type_num == TYPE_POINTER {
+                return Err(self.invalid_db_error("pointer points to another pointer"));
+            }
+            if type_num == expected_type {
+                return Ok(size);
+            }
+            return Err(self.decode_error(&format!("expected {label}, got type {type_num}")));
+        }
+        if type_num == expected_type {
             Ok(size)
         } else {
-            Err(self.decode_error(&format!("expected array, got type {type_num}")))
+            Err(self.decode_error(&format!("expected {label}, got type {type_num}")))
         }
     }
 
     /// Gets size and type, following any pointers.
     fn size_and_type_following_pointers(&mut self) -> DecodeResult<(usize, u8)> {
-        let (size, type_num) = self.size_and_type();
-        if type_num == TYPE_POINTER {
-            // Pointer - follow it
-            let new_ptr = self.decode_pointer(size);
-            self.current_ptr = new_ptr;
-            self.size_and_type_following_pointers()
-        } else {
-            Ok((size, type_num))
+        let (size, type_num) = self.size_and_type()?;
+        if type_num != TYPE_POINTER {
+            return Ok((size, type_num));
         }
+
+        self.current_ptr = self.decode_pointer(size);
+        let (size, type_num) = self.size_and_type()?;
+        if type_num == TYPE_POINTER {
+            return Err(self.invalid_db_error("pointer points to another pointer"));
+        }
+
+        Ok((size, type_num))
+    }
+
+    #[inline(always)]
+    fn read_string_bytes(&mut self, size: usize) -> DecodeResult<&'de [u8]> {
+        let new_offset = self
+            .current_ptr
+            .checked_add(size)
+            .ok_or_else(|| self.invalid_db_error("string length exceeds buffer"))?;
+        if new_offset > self.buf.len() {
+            return Err(self.invalid_db_error("string length exceeds buffer"));
+        }
+        let bytes = &self.buf[self.current_ptr..new_offset];
+        self.current_ptr = new_offset;
+        Ok(bytes)
     }
 
     /// Reads a string's bytes directly, following pointers if needed.
     /// Does NOT validate UTF-8.
     pub(crate) fn read_str_as_bytes(&mut self) -> DecodeResult<&'de [u8]> {
-        let (size, type_num) = self.size_and_type();
-        if type_num == TYPE_POINTER {
-            // Pointer
-            let new_ptr = self.decode_pointer(size);
-            let saved_ptr = self.current_ptr;
-            self.current_ptr = new_ptr;
-            let result = self.read_str_as_bytes();
-            self.current_ptr = saved_ptr;
-            result
-        } else if type_num == TYPE_STRING {
-            let new_offset = self.current_ptr + size;
-            if new_offset > self.buf.len() {
-                return Err(self.invalid_db_error("string length exceeds buffer"));
+        let (size, type_num) = self.size_and_type()?;
+        match type_num {
+            TYPE_POINTER => {
+                let new_ptr = self.decode_pointer(size);
+                let saved_ptr = self.current_ptr;
+                self.current_ptr = new_ptr;
+                let (size, type_num) = self.size_and_type()?;
+                let result = if type_num == TYPE_POINTER {
+                    Err(self.invalid_db_error("pointer points to another pointer"))
+                } else if type_num == TYPE_STRING {
+                    self.read_string_bytes(size)
+                } else {
+                    Err(self.invalid_db_error(&format!("expected string, got type {type_num}")))
+                };
+                self.current_ptr = saved_ptr;
+                result
+            }
+            TYPE_STRING => self.read_string_bytes(size),
+            _ => Err(self.invalid_db_error(&format!("expected string, got type {type_num}"))),
+        }
+    }
+
+    /// Fast-path identifier decoding:
+    /// - Returns `Ok(Some(bytes))` and consumes the identifier when it is a string.
+    /// - Returns `Ok(None)` and restores `current_ptr` when the next value is not a string.
+    /// - Returns `Err` for malformed pointer chains or invalid string lengths.
+    fn try_read_identifier_bytes(&mut self) -> DecodeResult<Option<&'de [u8]>> {
+        let saved_ptr = self.current_ptr;
+        let (size, type_num) = self.size_and_type()?;
+        match type_num {
+            TYPE_STRING => self.read_string_bytes(size).map(Some),
+            TYPE_POINTER => {
+                let new_ptr = self.decode_pointer(size);
+                let after_pointer = self.current_ptr;
+                self.current_ptr = new_ptr;
+                let (inner_size, inner_type) = self.size_and_type()?;
+                let result = if inner_type == TYPE_POINTER {
+                    Err(self.invalid_db_error("pointer points to another pointer"))
+                } else if inner_type == TYPE_STRING {
+                    self.read_string_bytes(inner_size).map(Some)
+                } else {
+                    Ok(None)
+                };
+                // decode_pointer(size) temporarily dereferences by moving current_ptr
+                // to new_ptr; after size_and_type/read_string_bytes on the pointed
+                // value, restoring current_ptr = after_pointer resumes parsing right
+                // after the original pointer bytes. When result is Ok(None), also
+                // reset current_ptr = saved_ptr so the non-string identifier can be
+                // parsed normally by the caller without consuming the pointer token.
+                self.current_ptr = after_pointer;
+                if matches!(result, Ok(None)) {
+                    self.current_ptr = saved_ptr;
+                }
+                result
+            }
+            _ => {
+                self.current_ptr = saved_ptr;
+                Ok(None)
             }
-            let bytes = &self.buf[self.current_ptr..new_offset];
-            self.current_ptr = new_offset;
-            Ok(bytes)
-        } else {
-            Err(self.invalid_db_error(&format!("expected string, got type {type_num}")))
         }
     }
 
     /// Skips the current value, following pointers.
     pub(crate) fn skip_value(&mut self) -> DecodeResult<()> {
-        let (size, type_num) = self.size_and_type();
+        let (size, type_num) = self.size_and_type()?;
         self.skip_value_inner(size, type_num, true)
     }
 
     /// Skips the current value without following pointers (for verification).
     pub(crate) fn skip_value_for_verification(&mut self) -> DecodeResult<()> {
-        let (size, type_num) = self.size_and_type();
+        let (size, type_num) = self.size_and_type()?;
         self.skip_value_inner(size, type_num, false)
     }
 
@@ -550,7 +656,7 @@ impl<'de> Decoder<'de> {
     }
 
     fn skip_value_inner_with_follow(&mut self, follow_pointers: bool) -> DecodeResult<()> {
-        let (size, type_num) = self.size_and_type();
+        let (size, type_num) = self.size_and_type()?;
         self.skip_value_inner(size, type_num, follow_pointers)
     }
 }
@@ -606,12 +712,9 @@ impl<'de: 'a, 'a> de::Deserializer<'de> for &'a mut Decoder<'de> {
     where
         V: Visitor<'de>,
     {
-        let (_, type_num) = self.peek_type()?;
-        if type_num == TYPE_STRING {
-            let bytes = self.read_str_as_bytes()?;
-            visitor.visit_borrowed_bytes(bytes)
-        } else {
-            self.decode_any(visitor)
+        match self.try_read_identifier_bytes()? {
+            Some(bytes) => visitor.visit_borrowed_bytes(bytes),
+            None => self.decode_any(visitor),
         }
     }
 
